# -*- coding: utf-8 -*-
"""
Created on Tue Feb 11 09:04:51 2020

@author: zszubbocsev
"""
"""
Created on Wed Jan  8 10:26:00 2020

@author: zszubbocsev
"""
 
 
import sys
import os
import http.client
import json
import ssl
import authenta.kms_proxy.crypto_utils as cr
from   Cryptodome.PublicKey import RSA
from   Cryptodome.Cipher import AES
from   Cryptodome.Cipher import PKCS1_OAEP
from   Cryptodome.Hash import SHA256
from   Cryptodome.Signature import pss

from timeit import default_timer as timer
from datetime import timedelta
import logging

logger = logging.getLogger(__name__)

# TODO The KMS Proxy server IP should go into a configuration file
#KMSProxyServerBaseAddress = 'microndeviceproxyapitest.micron.com'
#KMSProxyServerBaseAddress = '137.201.240.39'
#KMSProxyServerBaseAddress = 'bowauthtest01.micron.com'
#KMSProxyServerBaseAddress = '137.201.240.136'
#KMSProxyServerBaseAddress = 'bowauthprod01.micron.com'
#KMSProxyServerBaseAddress = '137.201.240.137'
#KMSProxyServerBaseAddress = 'authentaproxydev.micron.com'
KMSProxyServerBaseAddress = '137.201.240.75'
KMSProxyServerBaseURL = 'https://' + KMSProxyServerBaseAddress

MPNValues = {
    'REQUEST_SRK':             "MTANXSBB01",
    'REQUEST_BATCH_KEYS':      "MTANXSBB01B",
    'CONFIRM_DEVICE_IDENTITY': "MTANXOBD01",
    'RECOVER_DEVICE':          "MTANXOBG01",
    'REPLACE_SRK':             "MTANXSBC01"
}

def issue_post(connection,request_url,request_headers,request_body_dict):
    '''
        send a post request using the input connection on the 
        input URL using the input headers and body

        Arguments:
            connection        a valid (HTTPS) connection to the KMS proxy server
                              must support request() and getResponse()
            request_url       the URL to post to
            request_headers   the headers to use in the post
            request_body_dict the body to use in the post

        Return:
            response to the connection.getResponse()
            None on failure
    '''
    try:
        logger.info('ISSUE POST:')
        logger.info('  method: POST')
        logger.info('  url: ' + request_url)
        logger.info('  headers: {}'.format(request_headers))
        logger.info('  body: ' + json.dumps(request_body_dict))
        connection.request(method="POST", url=request_url, headers=request_headers, body=json.dumps(request_body_dict))
        response = connection.getresponse()
        logger.info('Response: Status = {}; Reason: {}'.format(response.status, response.reason))
    except:
        e = sys.exc_info()
        if e[0] is not None and e[1] is not None:
            logger.error("{}: {}".format(e[0], e[1]))
        elif e[0] is not None:
            logger.error(e[0])
        else:
            logger.error(e)
        response = None

    return response


def getKey(UID, Connection, nonce, pub_key_pem_str):
    """
    # Arguments
    #       UID       uid of the device
    #       Connetion a connected https connection object to the KMS proxy
    #       nonce     the nonce retrieved from the KMS for this session
    #
    #       pub_key_pem_str the RSA public key formatted as a pem string
    #
    # Return
    #        on success [encTransportKey (str), initialization vector (str), authentication tag (str), encrypted SRK value (str), 'OK', http_status_code]
    #        on error   [None, None, None, None, 'error message', http_status_code]
    #
    """
    # Issue GetKey 
    
    # Defining parts of the HTTP request
    request_url=KMSProxyServerBaseURL + '/GetKey'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    request_body_dict={
        'mpn': MPNValues['REQUEST_SRK'],
        'UID': UID,
        'keyType':"SRK",
        'wrapKey': pub_key_pem_str,
        'nonce': nonce,
        'wrapProtocol': "rsaEncryptedTransportKey",
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection,request_url,request_headers,request_body_dict)
    if response is None:
        logger.error("Post to GetKey on KMS Proxy failed!")
        return None, None, None, None, "Post to GetKey on KMS Proxy failed!", 0

    
    responsedata = response.read()
    if responsedata is not None and len(responsedata) > 0:
        try:
            received = json.loads(if_bytes_make_str(responsedata))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(responsedata))
            return None, None, None, None, 'KMS Returned status {}; Expected JSON in GetKey response instead received: {}'.format(response.status, responsedata). response.status

        logger.info(received)

    else:
        logger.error("Post to GetKey on KMS Proxy returned {} with empty message body!".format(response.status))
        return None, None, None, None, "GetKey on KMS Proxy returned {} with empty message body!".format(response.status), response.status

    if response.status < 200 or response.status >= 300:
        logger.error('Response: status = {}; body = {}'.format(response.status, received))
        return None, None, None, None, 'getKey Response: status = {}; body = {}'.format(response.status, received), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return received['encTransportKey'], received['iv'], received['tag'], received['ciphertext'], "OK", response.status

    

def VerifyRequestUIDResponse(UID, UIDData, Connection, noncetag, MPN, mtcPowerOnValue='false'):
    """
    ###################### Issue VerifyRequestUIDResponse #############################
    # Post a VerifyRequestUIDResponse to the 
    # Micron KMS proxy server and return the results
    #
    # Return
    #   bool (true if verified, false otherwise), error message, http response code
    """
    request_url=KMSProxyServerBaseURL + '/VerifyRequestUIDResponse'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    request_body_dict={
        'mpn': MPN,
        'UID': UID,
        'op':  "verifyRequestUIDResponse",
        'nonceTag': noncetag,
        'data': UIDData,
        'mtcPowerOnValue': mtcPowerOnValue,
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection,request_url,request_headers,request_body_dict)
    if response is None:
        logger.error("Post to verifyRequestUIDResponse on KMS Proxy failed!")
        return 0, "Post to verifyRequestUIDResponse on KMS Proxy failed!", 0
    data=response.read()

    if data is not None and len(data) > 0:
        try:
            received = json.loads(if_bytes_make_str(data))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(data))
            return 0, 'Expected JSON in getNonce response instead received: {}'.format(data), response.status

        logger.info(received)

    else:
        logger.error("Post to verifyRequestUIDResponse on KMS Proxy returned {} with empty message body!".format(response.status))
        return 0, "Post to verifyRequestUIDResponse on KMS Proxy returned {} with empty message body!".format(response.status), response.status

    if response.status < 200 or response.status >= 300:
        if received is not None:
            logger.error('Response: status = {}; body = {}'.format(response.status, received))
            return 0, 'Response: status = {}; body = {}'.format(response.status, received), response.status
        else:
            logger.error('Response: status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            return 0, 'Response: status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return 1, 'OK', response.status
   

def getNonceWithMPN(Connection, Mpn):
    """
    # Post GetNonce to the Micron KMS proxy server
    # 
    # Arguments:
    #   Connection  a valid connection to the KMS proxy server
    #   MPN the MPN str to send to the KMS for the task the returned nonce will be used for
    #           The MPN value must be recognized by the proxy server
    #
    # Returns on success: Nonce, Nonce-Tag (both str objects), http status code (int)
    #         on fail: None, 'error msg', http status code (int)
    """

    request_url=KMSProxyServerBaseURL + '/GetNonce'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    request_body_dict={
        'mpn': Mpn,
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection,request_url,request_headers,request_body_dict)    
    if response is None:
        logger.error("Post to GetNonce on KMS Proxy failed!")
        return None, "Post to GetNonce on KMS Proxy failed!", 0
    responsedata=response.read()
    if responsedata is not None and len(responsedata) > 0:
        try:
            received = json.loads(if_bytes_make_str(responsedata))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(responsedata))
            return None, 'Expected JSON in getNonce response instead received: {}'.format(responsedata), response.status

        logger.info(received)

    else:
        logger.error("Post to GetNonce on KMS Proxy returned {} ({}) with empty message body!".format(response.status, response.reason))
        return None, "GetNonce on KMS Proxy returned {} ({}) with empty message body!".format(response.status, response.reason), response.status

    if response.status < 200 or response.status >= 300:
        logger.error('Response: status = {}; body = {}'.format(response.status, received))
        return None, 'Response: status = {}; body = {}'.format(response.status, received), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return received['nonce'], received['nonceTag'], response.status

def GenerateCmdSignature(Op, CommandData, AdditionalData, Connection, Mpn, UID):
    """
    #   Post a GenerateCmdSignature to the KMS proxy
    #
    # Return on success, signature, "OK" (as str, str)
    #        on fail     None, "error-message"
    """
    request_url=KMSProxyServerBaseURL + '/GenerateCommandSignature'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    request_body_dict={
        'mpn':Mpn,
        'UID':UID,
        'op':  Op,
        'data': CommandData,
        'additionalData': AdditionalData,
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection, request_url, request_headers, request_body_dict)
    if response is None:
        logger.error("Post to GenerateCommandSignature on KMS Proxy failed!")
        return None, "Post to GenerateCommandSignature on KMS Proxy failed!", 0

    data=response.read()
    if data is not None and len(data) > 0:
        try:
            received = json.loads(if_bytes_make_str(data))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(data))
            return None, 'Expected JSON in GenerateCommandSignature response instead received: {}'.format(data), response.status

        logger.info(received)

    else:
        logger.error("Post to GenerateCommandSignature on KMS Proxy returned {} with empty message body!".format(response.status))
        return None, "GenerateCommandSignature on KMS Proxy returned {} with empty message body!".format(response.status), response.status
    
    if response.status < 200 or response.status >= 300:
        logger.error('Response: status = {}; body = {}'.format(response.status, received))
        return None, 'Response: status = {}; body = {}'.format(response.status, received), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return received['signature'], 'OK', response.status
    

def VerifyResponseSignature(Op, CommandData, Connection, Mpn, UID):
    """
    Post a VerifyResponseSig to the KSM proxy

    # ARGUMENTS
    #   Op operation 
    #
    # Return on success bool, 'OK'; 
    #                   True if signature in input response data is valid, False otherwise
    #        on errror None, 'error message'
    """
    request_url=KMSProxyServerBaseURL + '/VerifyResponseSig'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    
    request_body_dict={
        'mpn':Mpn,
        'UID':UID,
        'op':  Op,
        'data': CommandData,
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection,request_url,request_headers,request_body_dict)
    if response is None:
        logger.error("Post to VerifyResponseSig on KMS Proxy failed!")
        return None, "Post to VerifyResponseSig on KMS Proxy failed!", 0
    data=response.read()

    if data is not None and len(data) > 0:
        try:
            received = json.loads(if_bytes_make_str(data))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(data))
            return None, 'Expected JSON in GenerateCommandSignature response instead received: {}'.format(data), response.status

        logger.info(received)

    else:
        logger.error("Post to VerifyResponseSig on KMS Proxy returned {} with empty message body!".format(response.status))
        return None, "VerifyResponseSig on KMS Proxy returned {} with empty message body!".format(response.status), response.status

    if response.status == 400 and response.code == 7000: # invalid signature
        logger.info('Response: status = {}; {}'.format(response.status, received))
        return 0, "OK"
    elif response.status < 200 or response.status >= 300:
        logger.error('Response: status = {}; body = {}'.format(response.status, received))
        return None, 'Response: status = {}; body = {}'.format(response.status, received), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return 1, 'OK', response.status


def getKeys(batchID, batchType, Connection, pageId, pageSize, pub_key_pem_str):
    """
    # Argumentsdd
    #       batchID    the Micron lot ID of the batch of the devices 
    #                  (str)
    #       batchType  identifies the grouping of keys;
    #                  currently supported: "lotId" and "cbmId" 
    #                  (str)
    #       Connetion  a connected https connection object to the KMS proxy
    #       pageId     Batch retrieval uses a paging technique to return large data sets.
    #                  pageId is used to manage paged results: 
    #                    - When retrieving the first page of results, the value of this 
    #                      parameter must be 0. 
    #                    - When a page of results is returned, the return structure will 
    #                      include the pageId value. To retrieve the next page of results 
    #                      the function will be called again, providing the current pageId
    #                      value to identify the session. 
    #                    - Providing a value of 0 while a page session is active will 
    #                      start a new session and return the first page of results
    #                    - Page sessions will time out after 10 seconds with no access
    #                    - Page sessions will be deleted when the last results are
    #                      delivered to the client
    #                  (int)
    #       pageSize   indicates the number of records to return each time this function is
    #                  called. The maximum page size is 3200 for Naxos and 1600 for Indigo
    #                  (int) 
    #
    #       pub_key_filename the public key formatted as a pem string
    #
    # Return
    #        on success [encTransportKey (str), 
    #                    list of keys in with each key is a dictionary specified as follows:
    #                         { UID, iv, tag, ciphertext }
    #                         where UID: <hex-string formatted UID>
    #                               iv:  <initialization vector (str)>
    #                               tag: <authentication tag (str)>
    #                               ciphertext: <encrypted SRK value (str)>
    #                    pageId (int),
    #                    eod (bool)
    #                    'OK',
    #                    http_status_code]
    #        on error   [None, None, None, None, 'error message', http_status_code]
    #
    """
    # Issue GetKey 
    
    # Defining parts of the HTTP request
    request_url=KMSProxyServerBaseURL + '/GetKeys'
    
    request_headers = {
        'Content-Type': 'application/json'
    }
    request_body_dict={
        'mpn': MPNValues['REQUEST_BATCH_KEYS'],
        'batchId': batchID,
        'batchType': batchType,
        'keyType':"SRK",
        'pageId': pageId,
        'pageSize': pageSize,
        'wrapKey': pub_key_pem_str,
        'wrapProtocol': "rsaEncryptedTransportKey",
        'manufacturer': 'Micron'
    }
    
    response=issue_post(Connection,request_url,request_headers,request_body_dict)
    if response is None:
        logger.error("Post to GetKey on KMS Proxy failed!")
        return None, None, None, None, "Post to GetKey on KMS Proxy failed!", 0

    
    responsedata = response.read()
    if responsedata is not None and len(responsedata) > 0:
        try:
            received = json.loads(if_bytes_make_str(responsedata))
        except json.decoder.JSONDecodeError as x:
            logger.error('Failed to parse response as JSON! Status = {}; code = {}; reason = {}'.format(response.status, response.code, response.reason))
            logger.error('Response:  {}'.format(responsedata))
            return None, None, None, None, 'KMS Returned status {}; Expected JSON in GetKey response instead received: {}'.format(response.status, responsedata). response.status

        logger.info(received)

    else:
        logger.error("Post to GetKey on KMS Proxy returned {} with empty message body!".format(response.status))
        return None, None, None, None, "GetKey on KMS Proxy returned {} with empty message body!".format(response.status), response.status

    if response.status < 200 or response.status >= 300:
        logger.error('Response: status = {}; body = {}'.format(response.status, received))
        return None, None, None, None, 'getKey Response: status = {}; body = {}'.format(response.status, received), response.status
    else:
        logger.info('Response: status = {}; {}'.format(response.status, received))

    return received['encTransportKey'], received['keys'], received['pageId'], received['eod'], "OK", response.status

    

def createMTTLS(ClientCert, CertSecret):
    """
    ###################### Build Mutual TLS to Proxy Server ##########################
    # Defining certificate related stuff and host of endpoint
    #
    """
    certificate_file = ClientCert
    certificate_secret= CertSecret
    host = KMSProxyServerBaseAddress
    
    # Define the client certificate settings for https connection
    context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
    context.load_cert_chain(certfile=certificate_file, password=certificate_secret)
    
    # Create a connection to submit HTTP requests
    connection = None
    try:
        connection = http.client.HTTPSConnection(host, port=443, context=context)
    except:
        logger.error('Attempt to establish an HTTPSConnection failed',sys.exc_info()[0])
        
    return connection

def if_bytes_make_str(response_data):
    """
    depending on the Python version response.read() may return bytes or
    str.  If it is a bytes convert and return as a UTF-8 str.
    """
    if(isinstance(response_data, bytes)):
        response_data = response_data.decode('utf-8')
    
    return response_data