import ctypes as ct

import os
from collections import namedtuple

# ---------------------------------------------
# Globals
# ---------------------------------------------

# table of SPI-NOR, Authenta Op, sub-Op codes
ACRTM = '9b'

# ---------------------------------------------
# Custom Exceptions
# ---------------------------------------------
# used in DEVICE.py as flag to signal device locking failed
class DeviceBusyError(Exception): pass

# ---------------------------------------------

class naxos_content(ct.Structure):
  _fields_ = [
      ("bytes", ct.c_ubyte * 100),
      ("nb_bytes", ct.c_int),
      ("fd", ct.c_int)
  ]


class flash_address(ct.Structure):
  _fields_ = [
      ("length", ct.c_int),
      ("address", ct.c_int),
      ("fd", ct.c_int)
  ]

class flash_write_content(ct.Structure):
  _fields_ = [
      ("address", flash_address),
      ("data", ct.c_ubyte * 256)
  ]

class flash_lock_config(ct.Structure):
  _fields_ = [
      ("address", flash_address),
      ("lockbit", ct.c_bool)
  ]

class device_descriptor(ct.Structure):
  _fields_ = [
      ("status", ct.c_int),
      ("fd", ct.c_int)
  ]

class device_info(ct.Structure):
  _fields_ = [
      ("path", ct.c_char_p),
      ("UID", ct.c_int),
      ("device_status", ct.c_int),
      ("device_model", ct.c_int)
  ]

class list_info(ct.Structure):
  _fields_ = [
      ("device_info_list", device_info * 16),
      ("listsize", ct.c_int)
  ]

class large_message_return(ct.Structure):
  _fields_ = [
      ("status", ct.c_int),
      ("buffer_size", ct.c_int),
      ("return_buf", ct.c_ubyte * 65536)
  ]

class message_return(ct.Structure):
  _fields_ = [
      ("status", ct.c_int),
      ("buffer_size", ct.c_int),
      ("return_buf", ct.c_ubyte * 260)
  ]

class t_authenta_context(ct.Structure):
  _fields_ = [
      ("ip_address", ct.c_char_p),
      ("returned_type", ct.POINTER(ct.c_int)),
      ("out_buf", ct.POINTER(message_return)),
      ("large_out_buf", ct.POINTER(large_message_return)),
      ("descriptor", ct.POINTER(device_descriptor)),
      ("devices_list", ct.POINTER(list_info))
  ]

def msg_buf():
  status = 1
  buf_sz = 260
  byte_buf = (ct.c_ubyte * buf_sz)()
  msg_struct = message_return(status, buf_sz, byte_buf)
  return ct.pointer(msg_struct)

def large_msg_buf():
  status = 1
  buf_sz = 65536
  byte_buf = (ct.c_ubyte * buf_sz)()
  msg_struct = large_message_return(status, buf_sz, byte_buf)
  return ct.pointer(msg_struct)
