# Key Driver Build Instructions
These are the instructions for building the client server for the BeagleBone.

## Notice
Python binaries and modules for nxp board are too large so they are not saved in the repo.
Other C libs for nxp board are saved in "./bin" (i.e., "key_driver.so", "libxml2.so.2" "libmbedcrypto.so.3"). No need to re-compile them.

## Prerequisites
On Debian Linux distributions, run the following commands to install requisite packages, and make our build directory:

```bash
mkdir -p build

sudo apt-get install -y \
  build-essential \
  libxml2-dev \
  make \
  python3
```

## Build mbedtls
Using the provided mbedtls library, `mbedtls/`, cloned from [the official mbedtls repository](https://github.com/ARMmbed/mbedtls/commits/mbedtls-2.13.0), build the `libmbedcrypto.so.3` file. It will build to the `build/` directory.

```bash
export SHARED=1
cd mbedtls
make lib
cp library/libmbedcrypto.so.3 ../build/
cd ..
```

Add the `libmbedcrypto.so.3` file to your `LD_LIBRARY_PATH`.

```bash
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${PWD}/build
```

## Build key_driver
Now that we have mbedtls built, we can build the key driver, which expects `libmbedcrypto.so.3` to exist.

```bash
cd key_driver
make
cp authenta_key_driver.so ../build/
make clean
cd ..
```
