#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import logging

from kms_load_test_config import *
from datetime import datetime
from kms_latency_profile_decorator import kms_latency_profile

import app_common_logging as app_logging

# --- Authenta imports ---
import authenta.kms_proxy.MicronKMSProxyAPI as MicronKMSProxyAPI

VERSION = 0.01

PROGRAM_DES = '''DESCRIPTION:
'''

EPILOG = '''


AUTHOR:
\tMarshall Shapiro

COPYRIGHT: Micron Technology, Inc.; 2020

'''

app_logging.setup_app_logging()
logger = logging.getLogger(__name__)

STATUS = True

MPN_REQUEST_SRK = "MTANXSBB01"
MPN_CONFIRM_DEVICE_IDENTITY = "MTANXOBD01"
MPN_RECOVER_DEVICE = "MTANXOBG01"
MPN_REPLACE_SRK = "MTANXSBC01"

func_lookup = { }

def get_additional_output_for_KMS_functions(func_name, result):
    # print(f'\n[get_additional_output_for_KMS_functions] {func_name} -> {result}\n')
    if func_name == 'get_nonce_from_kms':
        success = True if result[2] == 200 else False
        error_message = "OK" if success else "\"{}\"".format(result[1])
        status = result[2]
        mpn = result[3]
    elif func_name in ['provision_crtm', 'crtm_register_read', 'replace_srk', 'deprovision_crtm']:
        success = True if result[0] is not None else False
        error_message = f"\"{result[1]}\""
        status = result[2] if result[2] is not None else 400
        mpn = result[3]
    elif func_name == 'get_key':
        success = True if result[0] is not None else False
        error_message = f"\"{result[4]}\""
        status = result[5] if result[5] is not None else 400
        mpn = result[6]
    elif func_name == 'kms_sign_command_function':
        success = True if result[1] == 'OK' else False
        error_message = f"\"{result[1]}\""
        status = result[2] if result[2] is not None else 400
        mpn = result[3]
    elif func_name in ['fetch_key_from_kms',
                       'confirm_device_identity',
                       'recover_device',
                       'kms_replace_srk'
                       'confirm_device_identity_no_nonce_tag',
                       'confirm_device_identity_stale_nonce_tag',
                       'confirm_device_identity_invalid_nonce_tag',
                       'confirm_device_identity_signature_failure',
                       'confirm_device_identity_signature_missing']:
        return get_additional_output_for_KMS_functions(result[0], result[1:])
    elif func_name == 'meta_request_uid':
        success = False
        error_message = 'Naxos device error on meta_request_uid'
        status = 0
        mpn = result[0]
    elif func_name == 'verify_request_uid_from_kms':
        success = True if result[0] is not None and result[0] != 0 else False
        error_message = f"\"{result[1]}\""
        status = result[2]
        mpn = result[3]
    elif func_name == 'verify_response_signature':
        success = True if result[0] is not None else False
        error_message = f"\"{result[1]}\""
        status = result[2]
        mpn = result[3]
    elif func_name == 'encryption_lib':
        success = False
        error_message = f"\"{result[0]}\""
        status = 0
        mpn = result[1]
    else:
        success = False
        error_message = f'UNRECOGNIZED FUNCTION: {func_name}' 
        status = 0
        mpn = 'UNKNOWN'

    return f'{success},{error_message},{status},{mpn}'


@kms_latency_profile
def get_nonce_from_kms(connection, mpn):
    #
    # get a nonce from the Micron KMS Proxy
    #
    the_nonce, nonce_tag_msg, status = MicronKMSProxyAPI.getNonceWithMPN(connection, mpn)

    # success = True if status == 200 else False
    # error_message = None if success else response
    # check_status(success, error_message, datetime.now())

    return the_nonce, nonce_tag_msg, status, mpn

def get_nonce_from_kms_op(device, connection):
    return get_nonce_from_kms(connection, MPN_CONFIRM_DEVICE_IDENTITY)
func_lookup['get_nonce_from_kms_op'] =  get_nonce_from_kms_op


@kms_latency_profile
def verify_request_uid_from_kms(connection, device, nonce_tag, req_uid_response, mpn, mtcPowerOnValue):
    #
    # Read UID from KMS (passing the UID of the attached Naxos
    # device and the response buffer of the call to Request_UID)
    #
    return MicronKMSProxyAPI.VerifyRequestUIDResponse(device.uid_as_str, req_uid_response.hex(), connection, nonce_tag, mpn, mtcPowerOnValue) + (mpn,)

@kms_latency_profile
def get_key(connection, device, a_nonce, public_key):
    #
    # Read UID from KMS (passing the UID of the attached Naxos
    # device and the response buffer of the call to Request_UID)
    #
    return MicronKMSProxyAPI.getKey(device.uid_as_str, connection, a_nonce, public_key) + (MPN_REQUEST_SRK,)

@kms_latency_profile
def kms_sign_command_function(device, command: bytes, additional_data: bytes=None):
    """
    return cmd_sig, err_code, status, mpn
    """
    return device.kms_sign_command_function(device, command, additional_data)

@kms_latency_profile
def verify_response_signature(verify_cmd_op, resp_data, connection, mpn, uid):
    """
    return verified, rc_msg, status, mpn
    """
    return MicronKMSProxyAPI.VerifyResponseSignature(verify_cmd_op, resp_data, connection, mpn, uid) + (mpn,)
