///
/// @file platform_driver_server_svc_indigo.cc
///
/// @brief Micron Indigo SDK GRPC server Indigo services
///
/// @copyright (C) 2019 Micron Technology, Inc. All rights reserved.
///

#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <grpcpp/grpcpp.h>

#include "mmc.h"
#include "../platform_driver_server_ll.h"
#include "platform_driver_server_svc_indigo.h"

#define MMC_IOC_MULTI
#define MMC_BLOCK_SIZE 512

using namespace std;
using namespace authenta;

static uint8_t respBuff[MMC_BLOCK_SIZE];
static struct device_mt_info *devices_ll_head;

static int send_authenta_command(int fd, uint8_t *cmdBuff, uint8_t *respBuff, uint32_t blockCount);
static grpc::Status authentaCmd(
    const indigo_content *request,
    message_return *reply,
    uint32_t blockCount);
static void printLongHex(const uint8_t * const data, const uint32_t len,
    const uint8_t widthInBytes, const char *title);
static int mmc_read_multiple_blocks(int fd, __u32 addr, uint32_t block_cnt, uint8_t * respBuff);
static int send_stop(int fd, __u8 hpi, __u32 *resp);
static int send_status(int fd, __u32 *response);
static int mmc_set_block_count(int fd, __u32 block_cnt);
static int mmc_write_multiple_blocks(int fd, __u32 addr, uint32_t block_cnt, uint8_t * respBuff);

// RPC call implementations

// Request_UID
grpc::Status IndigoServiceImpl::Request_UID(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{  
    printf("Requst_UID...\n");
    return authentaCmd(request, reply, 1);
}

// Provision_CRTM
grpc::Status IndigoServiceImpl::Provision_CRTM(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("Provision_CRTM...\n");
    return authentaCmd(request, reply, 1);
}

// Deprovision_CRTM
grpc::Status IndigoServiceImpl::Deprovision_CRTM(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)   
{
    printf("Deprovision_CRTM...\n");
    return authentaCmd(request, reply, 1);
}

// APA_Modify
grpc::Status IndigoServiceImpl::APA_Modify(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("APA_Modify...\n");
    return authentaCmd(request, reply, 1);
}

// Flash_Extend
grpc::Status IndigoServiceImpl::Flash_Extend(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("Flash_Extend...\n");
    return authentaCmd(request, reply, 1);
}

// ACRTM_Update_Start
grpc::Status IndigoServiceImpl::ACRTM_Update_Start(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("ACRTM_Update_Start...\n");
    return authentaCmd(request, reply, 1);
}

// ACRTM_Update_Finish
grpc::Status IndigoServiceImpl::ACRTM_Update_Finish(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("ACRTM_Update_Finish...\n");
    return authentaCmd(request, reply, 1);
}

// CRTM_Register_Read
grpc::Status IndigoServiceImpl::CRTM_Register_Read(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("CRTM_Register_Read...\n");
    return authentaCmd(request, reply, 1);
}

// CRTM_Register_Write
grpc::Status IndigoServiceImpl::CRTM_Register_Write(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("CRTM_Register_Write...\n");
    return authentaCmd(request, reply, 1);
}

// Replace_SRK
grpc::Status IndigoServiceImpl::Replace_SRK(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("Replace_SRK...\n");
    return authentaCmd(request, reply, 1);
}

// Manage_KTS
grpc::Status IndigoServiceImpl::Manage_KTS(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply) 
{
    printf("Manage_KTS...\n");
    return authentaCmd(request, reply, 1);
}

// Measure
grpc::Status IndigoServiceImpl::Measure(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    // get block count
    uint32_t blockCount = request->nb_bytes() / MMC_BLOCK_SIZE;

    printf("Measure...\n");
    return authentaCmd(request, reply, blockCount);
}

// PCR_Extend
grpc::Status IndigoServiceImpl::PCR_Extend(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("PCR_Extend...\n");
    return authentaCmd(request, reply, 1);
}

// CreateSignature
grpc::Status IndigoServiceImpl::CreateSignature(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("CreateSignature...\n");
    return authentaCmd(request, reply, 1);
}

// VerifySignature
grpc::Status IndigoServiceImpl::VerifySignature(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("VerifySignature...\n");
    return authentaCmd(request, reply, 1);
}

// Request_Monotonic_Counter
grpc::Status IndigoServiceImpl::Request_Monotonic_Counter(
    grpc::ServerContext *context,
    const indigo_content *request,
    message_return *reply)
{
    printf("Request_Monotonic_Counter...\n");
    return authentaCmd(request, reply, 1);
}

#define MAX_TRIES 5

// Read_Multiple_Block
grpc::Status IndigoServiceImpl::Read_Multiple_Block(
    grpc::ServerContext *context,
    const flash_address *request,
    message_return *reply)
{
    reply->set_status(STATUS_ERROR);
    reply->set_buffer_size(0);

    int fd = request->fd();
    int addr = request->address();
    int length = request->length();
    int ret;

    // wait for device to recover
    uint32_t response = 0;
    uint32_t numTries = 0;
    while (response != 0x900 && numTries < MAX_TRIES)
    {
        ret = send_status(fd, &response);
        if (ret)
        {
            cout << "ERROR: Failed to get device status" << endl;
            return grpc::Status::OK;
        }
        sleep(1);
        numTries++;
    }
    if(numTries > 1)
    {
        cout << "Tried to call CMD 13 (before read) " << numTries << " time(s) " << endl;
        cout << hex << "Response is 0x" << response << dec << endl;

        if (numTries == MAX_TRIES) {
            send_stop(fd, 1, &response);
            cout << "Issued CMD 12" << endl;

            ret = send_status(fd, &response);
            if (ret)
            {
                cout << "ERROR: Failed to get device status" << endl;
                return grpc::Status::OK;
            }
            cout << "Issued CMD 13" << endl;
            cout << hex << "Response is 0x" << response << dec << endl;
        }
    }
    
    printf("Read_Multiple_Block...\n");

    if (length % MMC_BLOCK_SIZE)
    {
        printf("Length of data to read is not valid (must be a multiple of %d bytes)\n", 
            MMC_BLOCK_SIZE);
        return grpc::Status::OK;
    }

    int blockCnt = length / MMC_BLOCK_SIZE;

    //CMD23 (set block count)
    ret = mmc_set_block_count(fd, blockCnt | (1 << 31));
    
    if (ret)
    {
        return grpc::Status::OK;
    }

    uint8_t * buf = (uint8_t *) malloc(length);
    memset(buf, '@', length);

    // cmd18
    ret = mmc_read_multiple_blocks(fd, addr, blockCnt, buf);
    if (!ret)
    {
        reply->set_return_buf(buf, request->length());
        reply->set_buffer_size(request->length());
        reply->set_status(STATUS_OK);
    }
    
    free(buf);

    // wait for device to recover
    response = 0;
    numTries = 0;
    while (response != 0x900 && numTries < MAX_TRIES)
    {
        ret = send_status(fd, &response);
        if (ret)
        {
            cout << "ERROR: Failed to get device status" << endl;
            return grpc::Status::OK;
        }
        sleep(1);
        numTries++;
    }
    if(numTries > 1)
    {
        cout << "Tried to call CMD 13 (after read) " << numTries << " time(s) " << endl;
        cout << hex << "Response is 0x" << response << dec << endl;

        if (numTries == MAX_TRIES) {
            send_stop(fd, 1, &response);
            cout << "Issued CMD 12" << endl;

            ret = send_status(fd, &response);
            if (ret)
            {
                cout << "ERROR: Failed to get device status" << endl;
                return grpc::Status::OK;
            }
            cout << "Issued CMD 13" << endl;
            cout << hex << "Response is 0x" << response << dec << endl;
        }
    }

    return grpc::Status::OK;
}

// Write_Multiple_Block
grpc::Status IndigoServiceImpl::Write_Multiple_Block(
    grpc::ServerContext *context,
    const flash_write_content *request,
    status_type_msg *reply)
{
    int fd = request->address().fd();
    int addr = request->address().address();
    int length = request->address().length();
    reply->set_status(STATUS_ERROR);
    int ret;

    // wait for device to recover
    uint32_t response = 0;
    uint32_t numTries = 0;
    while (response != 0x900 && numTries < MAX_TRIES)
    {
        ret = send_status(fd, &response);
        if (ret)
        {
            cout << "ERROR: Failed to get device status" << endl;
            return grpc::Status::OK;
        }
        sleep(1);
        numTries++;
    }
    if(numTries > 1)
    {
        cout << "Tried to call CMD 13 (before write) " << numTries << " time(s) " << endl;
        cout << hex << "Response is 0x" << response << dec << endl;

        if (numTries == MAX_TRIES) {
            send_stop(fd, 1, &response);
            cout << "Issued CMD 12" << endl;

            ret = send_status(fd, &response);
            if (ret)
            {
                cout << "ERROR: Failed to get device status" << endl;
                return grpc::Status::OK;
            }
            cout << "Issued CMD 13" << endl;
            cout << hex << "Response is 0x" << response << dec << endl;
        }
    }

    printf("Write_Multiple_Block...\n");

    if (length % MMC_BLOCK_SIZE)
    {
        printf("Length of data to write is not valid (must be a multiple of %d bytes)\n", 
            MMC_BLOCK_SIZE);
        return grpc::Status::OK;
    }

    int blockCnt = length / MMC_BLOCK_SIZE;

    //CMD23 (set block count)
    ret = mmc_set_block_count(fd, blockCnt | (1 << 31));
    
    if (ret)
    {
        return grpc::Status::OK;
    }

    //cmd25
    ret = mmc_write_multiple_blocks(fd, addr, blockCnt, (uint8_t *) request->data().c_str());

    if (!ret)
    {
        reply->set_status(STATUS_OK);
    }
    
    // wait for device to recover
    response = 0;
    numTries = 0;
    while (response != 0x900 && numTries < MAX_TRIES)
    {
        ret = send_status(fd, &response);
        if (ret)
        {
            cout << "ERROR: Failed to get device status" << endl;
            return grpc::Status::OK;
        }
        sleep(1);
        numTries++;
    }
    if(numTries > 1)
    {
        cout << "Tried to call CMD 13 (after write) " << numTries << " time(s) " << endl;
        cout << hex << "Response is 0x" << response << dec << endl;

        if (numTries == MAX_TRIES) {
            send_stop(fd, 1, &response);
            cout << "Issued CMD 12" << endl;

            ret = send_status(fd, &response);
            if (ret)
            {
                cout << "ERROR: Failed to get device status" << endl;
                return grpc::Status::OK;
            }
            cout << "Issued CMD 13" << endl;
            cout << hex << "Response is 0x" << response << dec << endl;
        }
    }

    return grpc::Status::OK;
}

// Authenta_device_open
grpc::Status IndigoServiceImpl::Authenta_device_open(
    grpc::ServerContext *context,
    const string_msg *request,
    device_descriptor *reply)
{
    printf("Authenta_device_open...\n");
    
    int fd = -1;
    const string &path = request->value();
    reply->set_fd(fd);
    reply->set_status(DEVICE_STATUS_ERROR);
    
    // make sure the inputs are valid
    if (path == "")
    {
        printf("path is not valid\n");
        return grpc::Status::OK;
    }

    fd = open(path.c_str(), O_RDWR);
    if (fd < 0)
    {
        printf("failed opening %s\n", path.c_str());
        return grpc::Status::OK;
    }

    reply->set_fd(fd);
    reply->set_status(DEVICE_STATUS_IDLE);
    devices_ll_head = add_head(devices_ll_head, fd, path.c_str());

    return grpc::Status::OK;
}

// Authenta_device_close
grpc::Status IndigoServiceImpl::Authenta_device_close(
    grpc::ServerContext *context,
    const int_msg *request,
    status_type_msg *reply)
{
    printf("Authenta_device_close...\n");
    
    int fd = request->value();
    reply->set_status(STATUS_ERROR);

    if (close(fd) == -1)
    {
        return grpc::Status::OK;
    }

    devices_ll_head = remove_element(devices_ll_head, fd);

    reply->set_status(STATUS_OK);
    
    return grpc::Status::OK;
}

// Acquire_Deivce
grpc::Status IndigoServiceImpl::Acquire_Device(
    grpc::ServerContext *context,
    const int_msg *request,
    status_type_msg *reply)
{
    printf("Acquire_Device...\n");
    
    int fd = request->value();
    reply->set_status(STATUS_ERROR);

    ll_result acq_result = acquire_device(devices_ll_head, fd);

    if (acq_result == DEVICE_BUSY)
    {
        printf("DEVICE_BUSY\n");
        return grpc::Status::OK;
    }
    else if (acq_result == DEVICE_NOT_FOUND)
    {
        printf("DEVICE_NOT_FOUND\n");
        return grpc::Status::OK;
    }
    else if (acq_result != DEVICE_ACQUIRED)
    {
        printf("unknown behavior\n");
        return grpc::Status::OK;
    }
    
    reply->set_status(STATUS_OK);

    return grpc::Status::OK;
}

// Release_Deivce
grpc::Status IndigoServiceImpl::Release_Device(
    grpc::ServerContext *context,
    const int_msg *request,
    status_type_msg *reply)
{
    printf("Release_Device (Indigo)...\n");
    
    int fd = request->value();
    reply->set_status(STATUS_ERROR);

    ll_result rel_result = release_device(devices_ll_head, fd);

    if (rel_result == DEVICE_NOT_OWNED)
    {
        printf("DEVICE_NOT_OWNED\n");
        return grpc::Status::OK;
    }
    else if (rel_result == DEVICE_NOT_FOUND)
    {
        printf("DEVICE_NOT_FOUND\n");
        return grpc::Status::OK;
    }
    else if (rel_result != DEVICE_FREE)
    {
        printf("unknown behavior\n");
        return grpc::Status::OK;
    }

    reply->set_status(STATUS_OK);

    return grpc::Status::OK;
}

// Authenta_detect_devices
grpc::Status IndigoServiceImpl::Authenta_detect_devices(
    grpc::ServerContext *context,
    const void_msg *request,
    list_info *reply)
{
    printf("Authenta_detect_devices...\n");

    // TODO

    return grpc::Status::OK;
}

/**
 * Send authenta command with one block data, i.e. only the command
 * 
 * @param[in] fd File descriptor for open MMC device
 * @param[in] Buffer of data to send for command
 * @param[out] respBuff Response data are copied into this buffer. Must be allocated by caller.
 * @param[in] Block count. Typically 1 for commmands, or n for data
 * @return 0 for OK. Non-zero for error.
 */
int send_authenta_command(int fd, uint8_t *cmdBuff, uint8_t *respBuff, uint32_t blockCount)
{
    int ret = 0;
    uint32_t arg = 0;
	int cmdCount = 0;
	uint8_t value;
	uint8_t ext_csd[512];
    
    //send commands: a read oriented command
    struct mmc_ioc_multi_cmd *multi_cmd;

	// Allocate an over sized buffer until we know exactly how many commands we'll pack in
	multi_cmd = (mmc_ioc_multi_cmd *) calloc(1, sizeof(struct mmc_ioc_multi_cmd) +
				16 * sizeof(struct mmc_ioc_cmd));
	if (!multi_cmd) {
		perror("failed to allocate memory\n");
		goto out;
	}

	//CMD13 (send status)
	multi_cmd->cmds[cmdCount].opcode = MMC_SEND_STATUS;
	multi_cmd->cmds[cmdCount].arg = (1 << 16);
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_R1 | MMC_CMD_AC;
	cmdCount++;

	// Enable CRTM
    /* CMD8(send ext csd)..CMD6(SWITCH).. */

    //CMD8(send ext csd)
	memset(ext_csd, 0, sizeof(ext_csd));
	multi_cmd->cmds[cmdCount].write_flag = 0;
	multi_cmd->cmds[cmdCount].opcode = MMC_SEND_EXT_CSD;
	multi_cmd->cmds[cmdCount].arg = 0;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_ADTC;
	multi_cmd->cmds[cmdCount].blksz = 512;
	multi_cmd->cmds[cmdCount].blocks = 1;
	mmc_ioc_cmd_set_data(multi_cmd->cmds[cmdCount], ext_csd);
	cmdCount++;

    //CMD6
	//wirte ext csd, byte 75 bit 0 to 1,to enable a-CRTM
    value = 0x01 | ext_csd[75];
		
	multi_cmd->cmds[cmdCount].write_flag = 1;
	multi_cmd->cmds[cmdCount].opcode = MMC_SWITCH;
	multi_cmd->cmds[cmdCount].arg = (MMC_SWITCH_MODE_WRITE_BYTE << 24) |
			(EXT_CSD_VENDOR_SPECIFIC_75 << 16) |
			(value << 8) |
			EXT_CSD_CMD_SET_NORMAL;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1B | MMC_RSP_R1B | MMC_CMD_AC;
	cmdCount++;

    //CMD23(set block count)..CMD25(write block multiple)..CMD23(set block count)..CMD18(read block)
	
	/* cmd 23 with reliable write*/
	multi_cmd->cmds[cmdCount].write_flag = 1;
	multi_cmd->cmds[cmdCount].opcode = MMC_SET_BLOCK_COUNT;
	if(1 == blockCount)
	{
		multi_cmd->cmds[cmdCount].arg = 1; //number of block: 1, set 1<< 31 for reliable write
	}
	else
	{
		multi_cmd->cmds[cmdCount].arg = blockCount | (1 << 31); //number of block: 1, set 1<< 31 for reliable write
	}
	multi_cmd->cmds[cmdCount].blksz      = 512;
	multi_cmd->cmds[cmdCount].arg |= 1<< 31;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_AC;
	cmdCount++;

	/* cmd 25 */
	multi_cmd->cmds[cmdCount].write_flag = 1;
	multi_cmd->cmds[cmdCount].write_flag |= (1<<31);
	multi_cmd->cmds[cmdCount].opcode = MMC_WRITE_MULTIPLE_BLOCK;
	multi_cmd->cmds[cmdCount].arg = arg;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1  |MMC_RSP_R1 | MMC_CMD_ADTC;
	multi_cmd->cmds[cmdCount].blksz      = 512;
	if(1 == blockCount)
	{
		multi_cmd->cmds[cmdCount].blocks = 1;
	}
	else
	{
		multi_cmd->cmds[cmdCount].blocks = blockCount;
	}
	multi_cmd->cmds[cmdCount].postsleep_min_us = 500000;
	multi_cmd->cmds[cmdCount].postsleep_max_us = 1000000;
    mmc_ioc_cmd_set_data(multi_cmd->cmds[cmdCount], cmdBuff);
	cmdCount++;

  	/* cmd 23 without reliable write*/
  	multi_cmd->cmds[cmdCount].write_flag = 1;
  	multi_cmd->cmds[cmdCount].opcode = MMC_SET_BLOCK_COUNT;
  	multi_cmd->cmds[cmdCount].arg = 1; //number of block: 1, set 1<< 31 for reliable write
  	multi_cmd->cmds[cmdCount].blksz      = 512;
  	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_AC;
	cmdCount++;

  	/* cmd 18 */
    multi_cmd->cmds[cmdCount].write_flag = 0;
    multi_cmd->cmds[cmdCount].arg = 0;
    multi_cmd->cmds[cmdCount].opcode   = MMC_READ_MULTIPLE_BLOCK;
    multi_cmd->cmds[cmdCount].flags     = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_ADTC;
    multi_cmd->cmds[cmdCount].blksz = 512;
    multi_cmd->cmds[cmdCount].blocks   = 1;
	mmc_ioc_cmd_set_data(multi_cmd->cmds[cmdCount], respBuff);
	cmdCount++;

	// Disable CRTM   
	/* CMD8(send ext csd)..CMD6(SWITCH).. */

    //CMD8(send ext csd)
	memset(ext_csd, 0, sizeof(ext_csd));
	multi_cmd->cmds[cmdCount].write_flag = 0;
	multi_cmd->cmds[cmdCount].opcode = MMC_SEND_EXT_CSD;
	multi_cmd->cmds[cmdCount].arg = 0;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_ADTC;
	multi_cmd->cmds[cmdCount].blksz = 512;
	multi_cmd->cmds[cmdCount].blocks = 1;
	mmc_ioc_cmd_set_data(multi_cmd->cmds[cmdCount], ext_csd);
	cmdCount++;

    //CMD6
	//wirte ext csd, byte 75 bit 0 to 0,to disable a-CRTM
    value = 0xFE & ext_csd[75];
		
	multi_cmd->cmds[cmdCount].write_flag = 1;
	multi_cmd->cmds[cmdCount].opcode = MMC_SWITCH;
	multi_cmd->cmds[cmdCount].arg = (MMC_SWITCH_MODE_WRITE_BYTE << 24) |
			(EXT_CSD_VENDOR_SPECIFIC_75 << 16) |
			(value << 8) |
			EXT_CSD_CMD_SET_NORMAL;
	multi_cmd->cmds[cmdCount].flags = MMC_RSP_SPI_R1B | MMC_RSP_R1B | MMC_CMD_AC;
	cmdCount++;

	/* prepare multi_cmd to be sent */
	multi_cmd->num_of_cmds = cmdCount;


  	ret = ioctl(fd, MMC_IOC_MULTI_CMD, multi_cmd);
  	if (ret) {
  		perror("Multi-cmd ioctl");

  		goto out;
  	}

out:
	free(multi_cmd);

    return ret;
}

// run authenta command and get response
static grpc::Status authentaCmd(
    const indigo_content *request,
    message_return *reply,
    uint32_t blockCount)
{      
    int fd = request->fd();
    reply->set_status(STATUS_ERROR);
    reply->set_buffer_size(0);

    if (!check_control(devices_ll_head, fd))
    {
        printf("DEVICE_NOT_OWNED\n");
        return grpc::Status::OK;
    }

    // check cmd size
    uint32_t cmdSize = request->nb_bytes();
    if (cmdSize % MMC_BLOCK_SIZE || cmdSize / MMC_BLOCK_SIZE < 1)
    {
        printf("cmd size is not valid\n");
        return grpc::Status::OK;
    }

    int ret = send_authenta_command(fd, (uint8_t *) request->bytes().c_str(), respBuff, blockCount);
    if (ret)
    {
        printf("cmd failed\n");
    }
    else
    {
        printLongHex(respBuff, MMC_BLOCK_SIZE, 16, "cmd resp" );
                
        reply->set_status(STATUS_OK);
        reply->set_buffer_size(MMC_BLOCK_SIZE);
        reply->set_return_buf(respBuff, MMC_BLOCK_SIZE);
    }

    return grpc::Status::OK;
}

// Print a long string of bytes as hex in neatly-formatted column.
// param[in] data   The data to print
// param[in] len    Number of bytes in data
// param[in] widthInBytes   Number of bytes to display per row (i.e. 0, 8, 16, 23). Will actually display twice this number of characters on screen.
// param[in] title  Title to display. Optional. Set to NULL to skip.
// NOTE: Set width to 0 to use indefinite row lenghts.
static void printLongHex(const uint8_t * const data, const uint32_t len,
    const uint8_t widthInBytes, const char *title)
{
    if (NULL != title)
    {
        printf("%s:", title);
    }

    if (0 != widthInBytes)
    {
        printf("\r\n");
    }

    for (uint32_t index = 0; index < len; index++)
    {
        if ((0 != widthInBytes) &&
            (0 == (index % widthInBytes) &&
            (0 != index)))
        {
            printf("\r\n");
        }

        printf("%02X", data[index]);
    }

    printf("\r\n");
}

/* CMD18
 * read multiple blocks: addr 0
 */
static int mmc_read_multiple_blocks(int fd, __u32 addr, uint32_t block_cnt, uint8_t * respBuff)
{
    int ret = 0;
    struct mmc_ioc_cmd idata;

    memset(&idata, 0, sizeof(idata));
//    memset(respBuff, 0, RESP_BUFFER_SIZE);
    idata.write_flag = 0;
//    idata.write_flag |= (1<<31);
    idata.arg = addr;
    idata.opcode   = MMC_READ_MULTIPLE_BLOCK;
    idata.flags     = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_ADTC;
    idata.blksz = 512;
    idata.blocks   = block_cnt;
    mmc_ioc_cmd_set_data(idata, respBuff);

    ret = ioctl(fd, MMC_IOC_CMD, &idata);
    if (ret)
    {
        perror("ioctl");
        printf("CMD %d ( %s: %s) Failed\n", MMC_READ_MULTIPLE_BLOCK, __FILE__, __FUNCTION__);
        return ret;
    }

    return ret;
}

 /* 
 * CMD12 (send stop)
 */
static int send_stop(int fd, __u8 hpi, __u32 *resp)
{
	int ret = 0;
	struct mmc_ioc_cmd idata;

	memset(&idata, 0, sizeof(idata));
	idata.opcode = MMC_STOP_TRANSMISSION;
	idata.arg = ((hpi ? 1 : 0) << 16) | hpi;
	idata.flags = MMC_RSP_SPI_R1B | MMC_RSP_R1B | MMC_CMD_AC;

	ret = ioctl(fd, MMC_IOC_CMD, &idata);
	if(ret)
		perror("ioctl");

	if(resp)
		*resp = idata.response[0];

	return ret;
}

 /* 
 * CMD13 (send status)
 */
int send_status(int fd, __u32 *response)
{
	int ret = 0;
	struct mmc_ioc_cmd idata;

	memset(&idata, 0, sizeof(idata));
	idata.opcode = MMC_SEND_STATUS;
	idata.arg = (1 << 16);
	idata.flags = MMC_RSP_R1 | MMC_CMD_AC;

	ret = ioctl(fd, MMC_IOC_CMD, &idata);
	if (ret)
	perror("ioctl");

	*response = idata.response[0];

	return ret;
}

 /* CMD23 (set block count)
 *
 */
static int mmc_set_block_count(int fd, __u32 block_cnt)
{
    int ret = 0;
    struct mmc_ioc_cmd idata;

    memset(&idata, 0, sizeof(idata));
    idata.write_flag = 1;
    idata.opcode = MMC_SET_BLOCK_COUNT;
    idata.arg = block_cnt; //number of block: 1, set 1<< 31 for reliable write
    idata.blksz      = 512;
    //idata.arg |= 1<< 31;
    idata.flags = MMC_RSP_SPI_R1 | MMC_RSP_R1 | MMC_CMD_AC;

    ret = ioctl(fd, MMC_IOC_CMD, &idata);
    if (ret)
    {
        perror("ioctl");
        printf("CMD %d ( %s) Failed\n", MMC_SET_BLOCK_COUNT, __FILE__);
        return ret;
    }

    printf("response (cmd23): 0x");
    for (int i = 0; i < 4; i++)
    {
    	printf("%08x", idata.response[i]);
    }
    printf("\n");

    return ret;
}

/* CMD25 (write multiple blocks): reliable write
 */
static int mmc_write_multiple_blocks(int fd, __u32 addr, __u32 block_cnt, __u8 * cmdBuff)
{
    int ret = 0;
    struct mmc_ioc_cmd idata;

    memset(&idata, 0, sizeof(idata));
    idata.write_flag = 1;
    idata.write_flag |= (1<<31);
    idata.opcode = MMC_WRITE_MULTIPLE_BLOCK;
    idata.arg = addr;
    idata.flags = MMC_RSP_SPI_R1  |MMC_RSP_R1 | MMC_CMD_ADTC;
    idata.blksz      = 512;
    idata.blocks     = block_cnt;

	idata.data_timeout_ns = 4000000000;

    mmc_ioc_cmd_set_data(idata, cmdBuff);

    ret = ioctl(fd, MMC_IOC_CMD, &idata);
    if (ret)
    {
        perror("ioctl");
        printf("CMD %d ( %s : %s) Failed\n", MMC_WRITE_MULTIPLE_BLOCK, __FILE__, __FUNCTION__);
        return ret;
    }

    printf("response (cmd25): 0x");
    for (int i = 0; i < 4; i++)
    {
    	printf("%08x", idata.response[i]);
    }
    printf("\n");

    return ret;
}
