///
/// @file platform_driver_server_svc_naxos.h
///
/// @brief Micron SDK2 GRPC server Naxos services
///
/// @copyright (C) 2019 Micron Technology, Inc. All rights reserved.
///

#ifndef _PLATFORM_DRIVER_SERVER_SVC_NAXOS_H
#define _PLATFORM_DRIVER_SERVER_SVC_NAXOS_H

#include "../platform_driver.grpc.pb.h"

class NaxosServiceImpl final : public authenta::Naxos::Service
{
    grpc::Status Authenta_device_open(
        grpc::ServerContext *context,
        const authenta::string_msg *request,
        authenta::device_descriptor *reply) override;

    grpc::Status Authenta_device_close(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Authenta_device_acquire(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Authenta_device_release(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Authenta_detect_devices(
        grpc::ServerContext *context,
        const authenta::void_msg *request,
        authenta::list_info *reply) override;

    grpc::Status Write_Enable(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Write_Disable(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Read_General_Purpose_Readout_Register(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::message_return *reply) override;

    grpc::Status Request_UID(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Auth_Modify(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Provision_CRTM(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Deprovision_CRTM(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;    

    grpc::Status Auth_lock(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status CRTM_Register_Read(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status CRTM_Register_Write(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Flash_extend(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status Replace_SRK(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status ACRTM_Update_Start(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status ACRTM_Update_Finish(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override;      

    grpc::Status Measure(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override; 

    grpc::Status Request_Monotonic_Counter(
        grpc::ServerContext *context,
        const authenta::naxos_content *request,
        authenta::status_type_msg *reply) override; 

    grpc::Status Fast_Read(
        grpc::ServerContext *context,
        const authenta::flash_address *request,
        authenta::message_return *reply) override;

    grpc::Status Page_Program(
        grpc::ServerContext *context,
        const authenta::flash_write_content *request,
        authenta::status_type_msg *reply) override;

    grpc::Status DYB_Write(
        grpc::ServerContext *context,
        const authenta::flash_lock_config *request,
        authenta::status_type_msg *reply) override;

    grpc::Status PPB_Program(
        grpc::ServerContext *context,
        const authenta::flash_address *request,
        authenta::status_type_msg *reply) override;

    grpc::Status PPB_Erase(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status PPB_Lock_Bit_Write(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::status_type_msg *reply) override;

    grpc::Status DYB_Read(
        grpc::ServerContext *context,
        const authenta::flash_address *request,
        authenta::message_return *reply) override;

    grpc::Status PPB_Read(
        grpc::ServerContext *context,
        const authenta::flash_address *request,
        authenta::message_return *reply) override;

    grpc::Status PPB_Lock_Bit_Read(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::message_return *reply) override;

    grpc::Status Password_Read(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::message_return *reply) override;

    grpc::Status Read_Status_register(
        grpc::ServerContext *context,
        const authenta::int_msg *request,
        authenta::message_return *reply) override;

    grpc::Status Subsector_Erase(
        grpc::ServerContext *context,
        const authenta::flash_address *request,
        authenta::status_type_msg *reply) override;        
};

#endif // _PLATFORM_DRIVER_SERVER_SVC_NAXOS_H
