///
/// @file platform_driver_server.cc
///
/// @brief Micron SDK2 GRPC server main program
///
/// @copyright (C) 2019 Micron Technology, Inc. All rights reserved.
///

#include <iostream>
#include <memory>
#include <string>

#include <grpcpp/grpcpp.h>

#include "naxos/platform_driver_server_svc_naxos.h"
#include "indigo/platform_driver_server_svc_indigo.h"

using grpc::Server;
using grpc::ServerBuilder;

void RunServer() {
  std::string server_address("0.0.0.0:50051");
  NaxosServiceImpl naxosService;
  IndigoServiceImpl indigoService;

  ServerBuilder builder;
  // Listen on the given address without any authentication mechanism.
  builder.AddListeningPort(server_address, grpc::InsecureServerCredentials());
  // Register "service" as the instance through which we'll communicate with
  // clients. In this case it corresponds to an *synchronous* service.
  builder.RegisterService(&naxosService);
  builder.RegisterService(&indigoService);
  // Finally assemble the server.
  std::unique_ptr<Server> server(builder.BuildAndStart());
  std::cout << "Server listening on " << server_address << std::endl;

  // Wait for the server to shutdown. Note that some other thread must be
  // responsible for shutting down the server for this call to ever return.
  server->Wait();
}

int main(int argc, char** argv)
{
  std::cout << "**** Starting gPRC Server  *****" << std::endl;

  RunServer();

  return 0;
}
