///
/// @file platform_driver_server_ll.cc
///
/// @brief Micron SDK2 GRPC server device linked list management lib
///
/// @copyright (C) 2019 Micron Technology, Inc. All rights reserved.
///

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "platform_driver_server_ll.h"

static char *get_path_from_fd(struct device_mt_info *head, int fd)
{
    struct device_mt_info *current = head;
    while (current != NULL)
    {
        if (current->fd == fd)
        {
            return current->path;
        }
        current = current->next;
    }
    return NULL;
}

static int find_control_fd(struct device_mt_info *old_head, char *path)
{
    struct device_mt_info *current = old_head;
    while (current != NULL)
    {
        if (strcmp(current->path, path) == 0 && current->acquired == true)
            return current->fd;
        current = current->next;
    }
    return -1;
}

struct device_mt_info *add_head(
    struct device_mt_info *former_head,
    int nfd,
    const char *npath)
{
    if (strlen(npath) > 256)
    {
        printf("path length %zu exceeds limit of %d\n", strlen(npath), 256);
        return NULL;
    }
    printf("adding head with fd %d and path %s\n", nfd, npath);
    struct device_mt_info *new_head = (struct device_mt_info *)malloc(sizeof(struct device_mt_info));
    new_head->next = former_head;
    new_head->fd = nfd;
    new_head->acquired = false;
    strcpy(new_head->path, npath);
    return new_head;
}

struct device_mt_info *remove_element(
    struct device_mt_info *old_head,
    int fd)
{
    if (old_head == NULL)
    {
        return old_head;
    }
    struct device_mt_info *current = old_head;
    if (fd == old_head->fd)
    {
        current = old_head->next;
        free(old_head);
        return current;
    }

    struct device_mt_info *previous = current;
    current = current->next;

    while (current != NULL)
    {
        if (fd == current->fd)
        {
            previous->next = current->next;
            free(current);
            break;
        }
        previous = current;
        current = current->next;
    }
    return old_head;
}

ll_result acquire_device(struct device_mt_info *old_head, int fd)
{
    char *path = get_path_from_fd(old_head, fd);
    if (find_control_fd(old_head, path) != -1)
    {
        return DEVICE_BUSY;
    }
    struct device_mt_info *current = old_head;
    while (current != NULL)
    {
        if (current->fd == fd)
        {
            current->acquired = true;
            return DEVICE_ACQUIRED;
        }
        current = current->next;
    }
    return DEVICE_NOT_FOUND;
}

ll_result release_device(struct device_mt_info *old_head, int fd)
{
    struct device_mt_info *current = old_head;
    while (current != NULL)
    {
        if (current->fd == fd)
        {
            if (current->acquired == true)
            {
                current->acquired = false;
                return DEVICE_FREE;
            }
            return DEVICE_NOT_OWNED;
        }
        current = current->next;
    }
    return DEVICE_NOT_FOUND;
}

bool check_control(struct device_mt_info *old_head, int fd)
{
    struct device_mt_info *current = old_head;
    while (current != NULL)
    {
        if (current->fd == fd && current->acquired == true)
            return true;
        current = current->next;
    }
    return false;
}
