import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_crtm_register_read_setup(request):
    global device
    global provision_stats
    device = conftest.get_device(request)
    yield

def test_crtm_register_read_ecdsa_nonvolatile_positive():
    with device:    
        error_code, response_bytes = device.meta_crtm_register_read(b'\x80\x00', 0x0)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_crtm_register_read_hmac_nonvolatile_positive():
    with device:
        # nonvolatile read
        error_code, response_bytes = device.meta_crtm_register_read(b'\x00\x00', 0xAC)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

def test_crtm_register_read_ecdsa_volatile_positive():
    with device:    
        error_code, response_bytes = device.meta_crtm_register_read(b'\x80\x01', 0x20)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

def test_crtm_register_read_hmac_volatile_positive():
    with device:    
        error_code, response_bytes = device.meta_crtm_register_read(b'\x00\x01', 0x20)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

@pytest.mark.parametrize("config, expectation", [(0x0, pytest.raises(TypeError)), (b"\x80\x00", does_not_raise())])
def test_meta_crtm_register_read_config_type(config, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_crtm_register_read(config, 0x0)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)            

@pytest.mark.parametrize("crtm_addr, expectation", [(b"\x00", pytest.raises(TypeError)), (0x0, does_not_raise())])
def test_meta_crtm_register_read_crtm_addr_type(crtm_addr, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_crtm_register_read(b"\x00\x00", crtm_addr = crtm_addr)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)
