import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_flash_extend_setup(request):
    global device
    device = conftest.get_device(request)
    with device:
        err, _ = device.meta_acrtm_update_start(b'\x80\x00')
        err, _ = device.meta_crtm_register_write(b'\x80\x00', 0x0, b'\x00'*88+b'\x00\x00\x00\x01'+b'\x00'*36, b'\x00'*88+b'\x00\x00\x00\x00'+b'\x00'*36)
        err, _ = device.meta_acrtm_update_finish(b'\x80\x00')

    yield

def test_flash_extend_positive_option0_ecdsa():
    """
    Option #0
    SigType: ECDSA signature
    """
    with device:
        
        error_code, response_bytes = device.meta_flash_extend(b'\x80\x00')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option0_hmac():
    """
    Option #0
    SigType: HMAC signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x00\x00')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option1_ecdsa():
    """
    Option #1
    SigType: ECDSA signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x80\x01')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option1_hmac():
    """
    Option #1
    SigType: HMAC signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x00\x01')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option2_ecdsa():
    """
    Option #2
    SigType: ECDSA signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x80\x02')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option2_hmac():
    """
    Option #2
    SigType: HMAC signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x00\x02')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option3_ecdsa():
    """
    Option #3
    SigType: ECDSA signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x80\x03')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_flash_extend_positive_option3_hmac():
    """
    Option #3
    SigType: HMAC signature
    """
    with device:

        error_code, response_bytes = device.meta_flash_extend(b'\x00\x03')

        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

@pytest.mark.parametrize("config,expectation", [(43, pytest.raises(TypeError)), (b"\x00\x00", does_not_raise())])
def test_meta_flash_extend_config_type(config, expectation):
    with device:
        with expectation:
            error_code, _ = device.meta_flash_extend(config)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)
