import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_pcr_extend_setup(request):
    global device
    global provision_stats
    global input_digest
    device = conftest.get_device(request)
    input_digest = b'\x11'*32
    yield

def test_pcr_extend_ECDSA_signature_positive():
    with device:

        error_code, response_bytes = device.meta_pcr_extend(b'\x80\x00', 0x0, input_digest)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

def test_pcr_extend_HMAC_signature_positive():
    with device:

        error_code, response_bytes = device.meta_pcr_extend(b'\x00\x00', 0x0, input_digest)
        assert(error_code.esr_and_response_success)
        assert(error_code._sig_matched)

@pytest.mark.parametrize("config, expectation", [(0x0, pytest.raises(TypeError)), (b"\x80\x00", does_not_raise())])
def test_meta_pcr_extend_config_type(config, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_pcr_extend(config, 0x0, input_digest)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

@pytest.mark.parametrize("address, expectation", [(b"\x00", pytest.raises(TypeError)), (0x0, does_not_raise())])
def test_meta_pcr_extend_address_type(address, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_pcr_extend(b"\x80\x00", address, input_digest)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

@pytest.mark.parametrize("input_digest, expectation", [(0x0, pytest.raises(TypeError)), (b'\x11'*32, does_not_raise())])
def test_meta_pcr_extend_input_digest_type(input_digest, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_pcr_extend(b"\x80\x00", 0x0, input_digest)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

@pytest.mark.parametrize("input_digest, expectation", [(b'\x11', pytest.raises(ValueError)), (b'\x11'*32, does_not_raise())])
def test_meta_pcr_extend_input_digest_size(input_digest, expectation):
    with expectation:
        with device:
            error_code, _ = device.meta_pcr_extend(b"\x80\x00", 0x0, input_digest)
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)
