import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_provision_crtm_setup(request):
    global device
    global provision_stats
    device = conftest.get_device(request)
    yield

def test_provision_crtm_remote_provision_positive():
    with device:

        err, _ = device.meta_request_uid()
        assert(err.esr_and_response_success)

        if device.provision_sts_struct.Provisioned:
            error_code, response = device.meta_deprovision_crtm(b'\x80\x00')
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

        err, response = device.meta_provision_crtm(b'\x80\x05')
        assert(err.esr_and_response_success)
        assert(err._sig_matched)

        # Assert Return Types
        assert isinstance(err, IndigoCmdErrorCode)
        assert isinstance(response, bytes)

@pytest.mark.parametrize("config, expectation", [(0x0, pytest.raises(TypeError)), (b"\x80\x05", does_not_raise())])
def test_provision_crtm_config_type(config, expectation):
    with device:
        with expectation:
            err, _ = device.meta_request_uid()
            assert(err.esr_and_response_success)

            if device.provision_sts_struct.Provisioned:
                error_code, response = device.meta_deprovision_crtm(b'\x80\x00')
                assert(error_code.esr_and_response_success)
                assert(error_code._sig_matched)

            error_code, _ = device.meta_provision_crtm(config)
            assert(err.esr_and_response_success)
            assert(err._sig_matched)

def test_provision_crtm_local_provision_positive():
    with device:

        err, _ = device.meta_request_uid()
        assert(err.esr_and_response_success)

        if device.provision_sts_struct.Provisioned:
            error_code, response = device.meta_deprovision_crtm(b'\x80\x00')
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

        err, response = device.meta_provision_crtm(b'\x80\x04')
        assert(err.esr_and_response_success)
        assert(err._sig_matched)

        # Assert Return Types
        assert isinstance(err, IndigoCmdErrorCode)
        assert isinstance(response, bytes)

def test_provision_crtm_local_provision_hsm_positive():
    with device:

        err, _ = device.meta_request_uid()
        assert(err.esr_and_response_success)

        if device.provision_sts_struct.Provisioned:
            error_code, response = device.meta_deprovision_crtm(b'\x80\x00')
            assert(error_code.esr_and_response_success)
            assert(error_code._sig_matched)

        err, response = device.meta_provision_crtm(b'\x80\x06')
        assert(err.esr_and_response_success)
        assert(err._sig_matched)

        # Assert Return Types
        assert isinstance(err, IndigoCmdErrorCode)
        assert isinstance(response, bytes)
