import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode
from authenta.grpc_platform import GrpcPlatform

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_read_write_multiple_block_setup(request):
    global device
    global provision_stats
    global crtm_addr
    global data
    device = conftest.get_device(request)
    crtm_addr = 0xf00000  
    data = b'\x12' * 512

    with device:
        err, _ = device.meta_request_uid()
        assert(err.esr_success)

    yield

def test_read_write_multiple_block_positive():
    with device:

        # Unlock address
        # APAWriteProtect = '0b
        # APAReadProtect = '0b
        apa_data = b'\x00'*80

        err, response = device.meta_apa_modify(b'\x80\x04', crtm_addr, apa_data, 0x0)
        assert(err.esr_and_response_success)
        assert(err._sig_matched)

        # Write data
        response = device.meta_write_multiple_block(address = crtm_addr, data = data)
        assert(response == True)

        # Read data
        ret, response = device.meta_read_multiple_block(crtm_addr, 0x200)
        assert(ret == True)
        assert(response == data)

@pytest.mark.parametrize("data, expectation", [(b'\x11', pytest.raises(ValueError))])
def test_write_multiple_block_data_input_lenght(data, expectation):
    with device:
        with expectation:
            ret, _  = device.meta_write_multiple_block(crtm_addr, data)
            assert(ret == True)

@pytest.mark.parametrize("length, expectation", [(0x1, pytest.raises(ValueError))])
def test_read_multiple_block_length_input_length(length, expectation):
    with device:
        with expectation:
            ret, _  = device.meta_read_multiple_block(crtm_addr, length)
            assert(ret == True)
