import sys
import os
import conftest
import pytest

from contextlib import contextmanager
from authenta.indigo.device import IndigoDevice, PlatformReturnType
from authenta.authenta_device import AuthentaDevice
from authenta.indigo.cmd_error_code import IndigoCmdErrorCode

@contextmanager
def does_not_raise():
    yield

@pytest.fixture(scope = "module", autouse = True)
def test_request_uid_setup(request):
    global device
    device = conftest.get_device(request)
    yield

def test_request_uid_positive():
    with device:
        error_code, response_bytes = device.meta_request_uid()
        assert(error_code.esr_and_response_success)

        # Assert Return Types
        assert isinstance(error_code, IndigoCmdErrorCode)
        assert isinstance(response_bytes, bytes)

@pytest.mark.parametrize("response_bytes, expectation", [(b'\x12\x34'*56, pytest.raises(ValueError)), (b'\x12\x34'*256, does_not_raise())])
def test_request_uid_response_bytes_length(response_bytes, expectation):
    with device:
        with expectation:
            device._request_uid_response_bytes(response_bytes)

@pytest.mark.parametrize("response_bytes, expectation", [(0x0, pytest.raises(TypeError)), (b'\x12\x34'*256, does_not_raise())])
def test_request_uid_response_bytes_type(response_bytes, expectation):
    with device:
        with expectation:
            device._request_uid_response_bytes(response_bytes)

@pytest.mark.parametrize("parsed_data, expectation", [(0x0, pytest.raises(TypeError))])
def test_request_uid_parse_type(parsed_data, expectation):
    with device:
        with expectation:
            device._request_uid_parse(parsed_data)

@pytest.mark.parametrize("config, expectation", [(0x0, pytest.raises(TypeError)), (b"\x00\x00", does_not_raise())])
def test_meta_request_uid_config_input_type(config, expectation):
    with device:
        with expectation:
            error_code, _ = device.meta_request_uid(config)
            assert(error_code.esr_and_response_success)
