#! /usr/bin/env python3

from authenta.naxos.device_1_5 import NaxosDevice_1_5
from authenta.grpc_platform import GrpcPlatform
from authenta.naxos.utils import CRTM_Registers
import authenta.naxos.utils as utils
import authenta.authenta_key_mgmt_interface as key_mgmt

dev_path = "/dev/spidev0.0"
ip_addr = "192.168.2.143"

ep = GrpcPlatform(dev_path, ip_addr)
device = NaxosDevice_1_5(ep)

with device:
  err, response, *_ = device.meta_request_uid()
  assert err.esr_and_response_success

  err, response = device.meta_request_monotonic_counter(0x80)
  assert err.esr_and_response_success

  err, response = device.meta_auth_lock()
  assert err.esr_and_response_success

  err, response = device.meta_acrtm_update_start(b"\x00")
  assert err.esr_and_response_success

  err, response = device.meta_crtm_register_write(b"\x00", CRTM_Registers.CRTM_Range_Start_Addr_0, b"\x00\x00\x00\x00", True)
  assert err.esr_and_response_success

  err, response = device.meta_crtm_register_write(b"\x00", CRTM_Registers.CRTM_Range_Start_Addr_0 + 4, b"\x00\x00\x10\x00", True)
  assert err.esr_and_response_success

  err, response = device.meta_acrtm_update_finish(b"\x00")
  assert err.esr_and_response_success

  err, response = device.meta_flash_extend(b"\x00")
  assert err.esr_and_response_success

  err, response = device.meta_measure(b"\x00\x00", 0, 0, 0)
  assert err.esr_and_response_success

  data = b"\x00" * 32

  for i in range(0, len(data), 4):
    addr = CRTM_Registers.KTR_Base + i
    err, response = device.meta_crtm_register_write(b"\x00", addr, data[i:(i + 4)], False)
    assert err.esr_and_response_success

  err, response = device.meta_crtm_register_read(b"\x00", CRTM_Registers.KTR_Base)
  assert err.esr_and_response_success

  new_key = key_mgmt.export_priv_key_as_hex(device.uid_as_str, 'test_key_handle')
  new_key_bytes = bytes.fromhex(new_key)
  new_key_hash = sha256(new_key_bytes).digest()
  err, response, *_ = device.meta_replace_srk(b"\x00", additional_data=new_key_hash)
  conftest.update_keys(device, uid, config, new_key_name)
  assert err.esr_success

  for i in range(0, len(data), 4):
    addr = CRTM_Registers.KTR_Base + i
    err, response = device.meta_crtm_register_write(b"\x00", addr, data[i:(i + 4)], False)
    assert err.esr_and_response_success

  new_key = key_mgmt.export_priv_key_as_hex(device.uid_as_str, 'srk_handle')
  new_key_bytes = bytes.fromhex(new_key)
  new_key_hash = sha256(new_key_bytes).digest()
  err, response, *_ = device.meta_replace_srk(b"\x01", additional_data=new_key_hash)
  conftest.update_keys(device, uid, config, new_key_name)
  assert err.esr_success

  # err, response = device.meta_deprovision_crtm(b"\x00")
  # assert err.esr_and_response_success

  # err, response, sent_nonce = device.meta_provision_crtm(b"\x01")
  # assert err.esr_and_response_success

  addr = 0xe00000

  # unlock block
  ret = device.meta_dyb_write(False, addr)
  assert ret

  # erase the data in specific block
  ret = device.meta_subsector_erase(addr)
  assert ret

  # write data to specific position
  data = b"\x11\x11"
  ret = device.meta_page_program(addr, data)
  assert ret

  # read data in block
  ret, data_read = device.meta_fast_read(addr)
  assert ret

  # lock block
  ret = device.meta_dyb_write(True, addr)
  assert ret
