#! /usr/bin/env python3

"""
API Test for Auth_Lock.
(C) 2019 Micron Technology, Inc. All rights reserved.
"""

import pytest

import authenta.naxos.utils as utils

import conftest

@pytest.fixture(scope="module", autouse=True)
def test_auth_lock_setup(request):
  global device
  global provision_stats

  device = conftest.get_device(request)
  with device:
    ret = utils.get_dev_info(device)
    assert(ret != None)
    (_, _, _, provision_stats) = ret

  yield

def test_auth_lock_Positive():
  if provision_stats.Provisioned == 0:
    return

  with device:
    err, _ = device.meta_auth_lock()
    assert(err.esr_and_response_success)

def test_auth_lock_Negative_NotProvisioned():
  """
  Not provisioned
  """
  if provision_stats.Provisioned == 0:
    with device:
      err, _ = device.meta_auth_lock()
      assert(err.esr == b"\x00")

def test_auth_lock_Negative_InvalidSig():
  """
  Invalid signature
  """
  if provision_stats.Provisioned == 0:
    return

  with device:
    device.test_only_set_boot_amtc0(b"\x00\x00\x00\x00")

    err, _ = device.meta_auth_lock()
    if err.esr != b"\x04":
      device.test_only_set_boot_amtc0(None)
      assert(False)

    device.test_only_set_boot_amtc0(None)

def test_auth_lock_Negative_InvalidCounter():
  """
  Invalid counter data
  """
  if provision_stats.Provisioned == 0:
    return

  with device:
    err, _ = device.read_extended_crtm_status()
    assert(err.esr_and_response_success)

    device.test_only_set_amtc0(b"\x00\x00\x00\x00")

    err, _ = device.meta_auth_lock()
    if err.esr != b"\x10":
      device.test_only_set_amtc0(None)
      assert(False)

    device.test_only_set_amtc0(None)