#! /usr/bin/env python3

"""
API Test for PPB_Program
(C) 2019 Micron Technology, Inc. All rights reserved.
"""

import pytest

import authenta.naxos.utils as utils

import conftest

@pytest.fixture(scope="module", autouse=True)
def test_ppb_program_setup(request):
  global device
  global provision_stats

  device = conftest.get_device(request)
  with device:
    ret = utils.get_dev_info(device)
    assert(ret != None)
    (_, _, _, provision_stats) = ret

  yield

def test_ppb_program_Positive_Provisoned():
  if provision_stats.Provisioned == 1:
    with device:
      addr = 0xe40000

      # Auth_Lock
      err, _ = device.meta_auth_lock()
      assert(err.esr == b"\x80")

      ret = device.meta_ppb_program(addr)
      assert(ret)

      ret = device.meta_ppb_erase()
      assert(ret)

def test_ppb_program_Positive_NotProvisoned():
  if provision_stats.Provisioned == 0:
    with device:
      addr = 0xe40000

      ret = device.meta_ppb_program(addr)
      assert(ret)

      ret = device.meta_ppb_erase()
      assert(ret)
