#! /usr/bin/env python3

"""
System test: amtc0.
(C) 2019 Micron Technology, Inc. All rights reserved.
"""

import sys
import os
import pytest

import authenta.naxos.utils as utils

import conftest

@pytest.fixture(scope="module", autouse=True)
def test_system_amtc0_setup(request):
  global device
  global iboot_ip
  global provision_stats

  device = conftest.get_device(request)
  iboot_ip = request.config.cache.get("iboot_ip", None)
  with device:
    ret = utils.get_dev_info(device)
    assert(ret != None)
    (_, _, _, provision_stats) = ret

  yield

def test_system_amtc0_Deprovision():
  """
  When device is deprovisioned, reboot should not change the amtc0 value
  """
  if provision_stats.Provisioned == 0:
    with device:
      ret = utils.get_dev_info(device)
      assert(ret != None)
      (_, amtc0, _, _) = ret

    ret = conftest.reboot(device, iboot_ip)
    assert(ret)

    with device:
      ret = utils.get_dev_info(device)
      assert(ret != None)
      (_, amtc0_new, _, _) = ret

      assert(amtc0 == amtc0_new)
