#! /usr/bin/env python3

"""
System Test: nonauthenticated command counter.
(C) 2019 Micron Technology, Inc. All rights reserved.
"""

import sys
import os
import pytest

import authenta.naxos.utils as utils
from authenta.naxos.utils import CRTM_Registers

import conftest

@pytest.fixture(scope="module", autouse=True)
def test_system_nonauthenticated_cmd_counter_setup(request):
  global device
  global provision_stats

  device = conftest.get_device(request)
  with device:
    ret = utils.get_dev_info(device)
    assert(ret != None)
    (_, _, _, provision_stats) = ret

  yield

def test_system_nonauthenticated_cmd_counter():
  """
  Call nonauthenticated Authenta command (e.g., CRTM_Register_Read) for > 255 times
  should cause an error and error can be cleared by an authentaicated Authenta command
  """
  if provision_stats.Provisioned == 0:
    return

  with device:
    # clear coutner first
    err, _ = device.meta_acrtm_update_finish(b"\x00")
    assert(err.esr_and_response_success)

    for _ in range(0, 255):
      err, _ = device.meta_crtm_register_read(b"\x00", CRTM_Registers.Device_Config)
    
    err, _ = device.meta_crtm_register_read(b"\x00", CRTM_Registers.Device_Config)
    assert(err.esr == b"\x04")

    err, _ = device.meta_acrtm_update_finish(b"\x00")
    assert(err.esr_and_response_success)

    err, _ = device.meta_crtm_register_read(b"\x00", CRTM_Registers.Device_Config)
    assert(err.esr_and_response_success)
