#! /usr/bin/env python3

"""
System test: provision status.
(C) 2019 Micron Technology, Inc. All rights reserved.
"""

import sys
import os
import pytest

import authenta.naxos.utils as utils
from authenta.naxos.utils import CRTM_Registers

import conftest

def reboot_and_check_provision_sts_provisioned(provision_sts_byte_std):
  # reboot 2 times
  reboot_times = 2
  for _ in range(0, reboot_times):
    ret = conftest.reboot(device, iboot_ip)
    assert(ret)

    # verify Provision_Sts
    with device:
      # Request_UID
      err, response, *_ = device.meta_request_uid()
      assert(err.esr_and_response_success)

      provision_sts_byte1 = response[3]
      assert(provision_sts_byte_std == provision_sts_byte1 & 0x7f)


def reboot_and_check_provision_sts_deprovisioned():
  ret = conftest.reboot(device, iboot_ip)
  assert(ret)

  # verify Provision_Sts
  with device:
    # Request_UID
    err, response, *_ = device.meta_request_uid()
    assert(err.esr_and_response_success)
    provision_sts_byte1 = response[3]
    assert(provision_sts_byte1 & 0xf0 == 0)

@pytest.fixture(scope="module", autouse=True)
def test_system_provision_status_setup(request):
  global device
  global iboot_ip
  global uid
  global provision_sts
  global provision_sts_byte

  device = conftest.get_device(request)
  iboot_ip = request.config.cache.get("iboot_ip", None)
  with device:
    err, response, *_ = device.meta_request_uid()
    assert(err.esr_and_response_success)

    provision_sts_byte = response[3]
    provision_sts = device.provision_sts_struct
    uid = device.uid

  yield

def test_system_provision_status_ProvisionThirdPartyKey():
  """
  if provisioned with 3rd party key, Provision_Sts should be stable after
  rebooting many times
  """
  if provision_sts.Remotely_Provisioned == 1 and provision_sts.SRK_Replaced == 1:
    # verify Provision_Sts
    reboot_and_check_provision_sts_provisioned(provision_sts_byte)

  elif provision_sts.Remotely_Provisioned == 1 and provision_sts.SRK_Replaced == 0:
    with device:
      # replace Micron SRK with a third-party SRK
      ret = conftest.replace_srk_util(device, uid, b"\x00", new_key_name='test_key_handle')
      assert(ret)

      # get updated Provsion_Sts
      err, response, *_ = device.meta_request_uid()
      assert(err.esr_and_response_success)
      provision_sts_bytes_new = response[3]

    # verify Provision_Sts after reboot
    reboot_and_check_provision_sts_provisioned(provision_sts_bytes_new)

    with device:
      # revert back to Micron SRK
      ret = conftest.replace_srk_util(device, uid, b"\x01", new_key_name='micron_srk_handle')
      assert(ret)

def test_system_provision_status_Deprovision():
  """
  if deprovisioned, Provision_Sts upper niblle should be always 0x0
  """
  if provision_sts.Provisioned == 0:
    assert(provision_sts_byte & 0xf0 == 0)
    reboot_and_check_provision_sts_deprovisioned()
